@@ -1,7 +1,7 @@
 /*
  * gtk_anyone.c: Show the user tree as a Gtk+ widget
  * David Maze <dmaze@mit.edu>
- * $Id: gtkanyone.c,v 1.15 2001/10/20 14:47:02 dmaze Exp $
+ * $Id: gtkanyone.c,v 1.22 2001/11/18 23:17:54 dmaze Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -9,28 +9,18 @@
 #endif
 
 #include "gtkanyone.h"
+#include "gtkzgroupitem.h"
+#include "gtkzuseritem.h"
 #include "anyone.h"
 
 /* Local function prototypes */
 static void gtk_anyone_class_init(GtkAnyoneClass *klass);
 static void gtk_anyone_init(GtkAnyone *anyone);
 
+static void build_children(GtkAnyone *anyone);
 static void rebuild_children(GtkAnyone *anyone);
 static void set_tooltip(GtkAnyone *anyone, GtkWidget *child, zuser *user);
 
-/* Populating the object
- * 
- * build_children() is called by gtk_anyone_construct(); it adds every
- * possible GtkTreeItem for both member users and child groups to this.
- * gtk_anyone_show_user() and gtk_anyone_hide_user() show and hide
- * relevant children.  This implementation may be changed, mostly because
- * of Gtk sucking (the tree looks wrong if the last visible item is not the
- * last item). */
-static void build_children(GtkAnyone *anyone);
-static void show_user_callback(GtkWidget *child, zuser *target);
-static void show_the_user(GtkWidget *child, zuser *user);
-static void hide_user_callback(GtkWidget *child, zuser *target);
-
 /* Get some help with updating */
 struct update_iter_data
 {
@@ -48,10 +38,8 @@ static void gtk_anyone_class_init(GtkAnyoneClass *klass)
 
 static void gtk_anyone_init(GtkAnyone *anyone)
 {
-  anyone->groupmap = NULL;
   anyone->group = NULL;
   anyone->filename = NULL;
-  anyone->children = NULL;
   anyone->tips = NULL;
 }
 
@@ -139,12 +127,9 @@ static void build_children(GtkAnyone *anyone)
   {
     /* Build the child group object... */
     group = list->data;
-    subtree = gtk_anyone_new(group);
 
     /* Build a GtkTreeItem for it... */
-    name = zgroup_label(group);
-    child = gtk_tree_item_new_with_label(name->str);
-    g_string_free(name, TRUE);
+    child = gtk_zgroup_item_new(group);
 
     /* Have to attach the child to the parent before we tweak things
      * like subtrees.  Remember, kids, documentation is for wusses, and
@@ -153,17 +138,20 @@ static void build_children(GtkAnyone *anyone)
 
     /* Make sure we show the children. */
     gtk_widget_show(GTK_WIDGET(child));
-    gtk_widget_show(GTK_WIDGET(subtree));
 
-    /* Attach the two, and deal with expansion and such. */
-    gtk_tree_item_set_subtree(GTK_TREE_ITEM(child), subtree);
-    if (zgroup_get_expanded(group))
-      gtk_tree_item_expand(GTK_TREE_ITEM(child));
-    else
-      gtk_tree_item_collapse(GTK_TREE_ITEM(child));
-
-    /* Save the child in our list of children. */
-    anyone->children = g_list_append(anyone->children, subtree);
+    /* Only create the tree now if the group has children of its own.
+    /* rebuild_children() will create groups for users if they don't
+     * already exist, to get around gtk+ brokenness. */
+    if (zgroup_groups(group))
+    {
+      subtree = gtk_anyone_new(group);
+      gtk_widget_show(GTK_WIDGET(subtree));
+      gtk_tree_item_set_subtree(GTK_TREE_ITEM(child), subtree);
+      if (zgroup_get_expanded(group))
+        gtk_tree_item_expand(GTK_TREE_ITEM(child));
+      else
+        gtk_tree_item_collapse(GTK_TREE_ITEM(child));
+    }
   }
 }
 
@@ -253,13 +241,12 @@ static void rebuild_children(GtkAnyone *anyone)
     GtkWidget *widget = GTK_WIDGET(lwidget->data);
     zuser *target, *user;
 
-    /* Sanity check: the widget should be a GtkTreeItem.  Stop if it
-     * has a subtree (since then we'll have reached the end of our own
-     * users). */
-    if (GTK_TREE_ITEM_SUBTREE(widget))
+    /* Sanity check: the widget should be a GtkZUserItem.  Stop if it
+     * isn't (since then we'll have reached the end of our own users). */
+    if (!IS_GTK_ZUSER_ITEM(widget))
       break;
 
-    target = (zuser *)gtk_object_get_user_data(GTK_OBJECT(widget));
+    target = gtk_zuser_item_user(GTK_ZUSER_ITEM(widget));
     user = luser->data;
 
     while (user != target)
@@ -268,16 +255,8 @@ static void rebuild_children(GtkAnyone *anyone)
       if (zuser_count_locs(user) > 0)
       {
         GtkWidget *child;
-        GString *name;
 
-        name = zuser_name(user);
-        child = gtk_tree_item_new_with_label(name->str);
-        g_string_free(name, TRUE);
-
-        /* Set its user data to the relevant zuser. */
-        gtk_object_set_user_data(GTK_OBJECT(child), user);
-    
-        /* Add the tree item to the tree. */
+        child = gtk_zuser_item_new(user);
         gtk_tree_insert(GTK_TREE(anyone), child, pos++);
         gtk_widget_show(child);
         set_tooltip(anyone, child, user);
@@ -316,16 +295,8 @@ static void rebuild_children(GtkAnyone *anyone)
       if (zuser_count_locs(user) > 0)
       {
         GtkWidget *child;
-        GString *name;
-        
-        name = zuser_name(user);
-        child = gtk_tree_item_new_with_label(name->str);
-        g_string_free(name, TRUE);
-        
-        /* Set its user data to the relevant zuser. */
-        gtk_object_set_user_data(GTK_OBJECT(child), user);
-        
-        /* Add the tree item to the tree. */
+
+        child = gtk_zuser_item_new(user);
         gtk_tree_insert(GTK_TREE(anyone), child, pos++);
         gtk_widget_show(child);
         set_tooltip(anyone, child, user);
@@ -335,8 +306,27 @@ static void rebuild_children(GtkAnyone *anyone)
     }
 
   /* Recurse into subgroups. */
-  for (lwidget = anyone->children; lwidget; lwidget = g_list_next(lwidget))
-    rebuild_children(GTK_ANYONE(lwidget->data));
+  for (; lwidget; lwidget = g_list_next(lwidget))
+  {
+    /* Every item now should be a group item. */
+    GtkZGroupItem *item = GTK_ZGROUP_ITEM(lwidget->data);
+    /* Check it for a child GtkAnyone object. */
+    GtkWidget *subtree = GTK_TREE_ITEM_SUBTREE(GTK_TREE_ITEM(item));
+    if (subtree == NULL)
+    {
+      /* Oops, we lost the subtree (or never had it). */
+      zgroup *group = gtk_zgroup_item_group(item);
+      subtree = gtk_anyone_new(group);
+      gtk_widget_show(subtree);
+      gtk_tree_item_set_subtree(GTK_TREE_ITEM(item), subtree);
+      if (zgroup_get_expanded(group))
+        gtk_tree_item_expand(GTK_TREE_ITEM(item));
+      else
+        gtk_tree_item_collapse(GTK_TREE_ITEM(item));
+    }
+    
+    rebuild_children(GTK_ANYONE(subtree));
+  }
 }
 
 static void set_tooltip(GtkAnyone *anyone, GtkWidget *child, zuser *user)
