@@ -1,7 +1,7 @@
 /*
  * gtkanyone.h: Show the user tree as a Gtk+ widget
  * David Maze <dmaze@mit.edu>
- * $Id: gtkanyone.h,v 1.12 2001/10/20 14:22:31 dmaze Exp $
+ * $Id: gtkanyone.h,v 1.14 2001/11/18 23:01:50 dmaze Exp $
  */
 
 #ifndef __GTKANYONE_H__
@@ -27,10 +27,8 @@ struct _GtkAnyone
 {
   GtkTree tree;
   
-  GHashTable *groupmap;
   zgroup *group;
   GString *filename;
-  GList *children;
   GtkTooltips *tips;
 };
 
