@@ -1,7 +1,7 @@
 /*
  * gznola.c: A GNOME-based xznola
  * David Maze <dmaze@mit.edu>
- * $Id: gznola.c,v 1.9 2000/04/18 20:01:12 dmaze Exp $
+ * $Id: gznola.c,v 1.12 2001/10/26 02:47:03 dmaze Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -41,6 +41,14 @@ void help_about(gpointer data);
 
 void do_file_open(gpointer data);
 void do_file_locate(gchar *string, gpointer data);
+void file_preferences_changed(GtkWidget *widget, gpointer data);
+void file_preferences_apply(GnomePropertyBox *propbox,
+                            gint page, gpointer data);
+
+void read_settings(void);
+void write_settings(void);
+void update_setup(void);
+gint update_timeout(gpointer data);
 
 /* Definition of the main menu */
 static GnomeUIInfo menu_file[] = {
@@ -93,6 +101,17 @@ static GnomeUIInfo toolbar[] = {
   GNOMEUIINFO_END
 };
 
+struct preferences
+{
+  int do_zsubs;
+  int do_time;
+  int time_interval;
+  int timer;
+} prefs;
+struct file_preferences_data
+{
+  GtkWidget *zsub_check, *time_check, *time_interval;
+};
 static GtkWidget *app, *progbar;
 static GtkAnyone *anyone;
 
@@ -103,6 +122,9 @@ int main(int argc, char **argv)
   ZInitialize();
   gnome_init(MYNAME, VERSION, argc, argv);
 
+  prefs.timer = 0;
+  read_settings();
+
   app = gnome_app_new(MYNAME, MYNAME);
   gnome_app_create_menus(GNOME_APP(app), menu_main);
   gnome_app_create_toolbar(GNOME_APP(app), toolbar);
@@ -119,6 +141,7 @@ int main(int argc, char **argv)
   g_string_free(file, TRUE);
 
   znol_update();
+  update_setup();
   
   gtk_main();
   return 0;
@@ -297,6 +320,91 @@ void do_file_locate(gchar *string, gpointer data)
  */
 void file_preferences(gpointer data)
 {
+  /* Create the preferences dialog. */
+  GtkWidget *dialog, *vbox, *hbox, *label;
+  struct file_preferences_data d;
+  gchar *interval;
+
+  dialog = gnome_property_box_new();
+  gnome_dialog_set_parent(GNOME_DIALOG(dialog), GTK_WINDOW(app));
+
+  vbox = gtk_vbox_new(FALSE, GNOME_PAD);
+  
+  d.zsub_check =
+    gtk_check_button_new_with_label("Update using zephyr subscriptions");
+  gtk_object_ref(GTK_OBJECT(d.zsub_check));
+  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(d.zsub_check),
+                               prefs.do_zsubs);
+  gtk_box_pack_start_defaults(GTK_BOX(vbox), d.zsub_check);
+  gtk_signal_connect(GTK_OBJECT(d.zsub_check), "toggled",
+                     GTK_SIGNAL_FUNC(file_preferences_changed), dialog);
+  /* Until we actually do this. */
+  gtk_widget_set_sensitive(d.zsub_check, FALSE);
+
+  hbox = gtk_hbox_new(FALSE, 0);
+  label = gtk_label_new("Update every ");
+  gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
+  d.time_interval = gtk_entry_new();
+  gtk_object_ref(GTK_OBJECT(d.time_interval));
+  interval = g_strdup_printf("%d", prefs.time_interval);
+  gtk_entry_set_text(GTK_ENTRY(d.time_interval), interval);
+  g_free(interval);
+  gtk_box_pack_start(GTK_BOX(hbox), d.time_interval, FALSE, FALSE, 0);
+  gtk_signal_connect(GTK_OBJECT(d.time_interval), "changed",
+                     GTK_SIGNAL_FUNC(file_preferences_changed), dialog);
+  label = gtk_label_new(" seconds");
+  gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
+
+  d.time_check = gtk_check_button_new();
+  gtk_object_ref(GTK_OBJECT(d.time_check));
+  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(d.time_check), prefs.do_time);
+  gtk_container_add(GTK_CONTAINER(d.time_check), hbox);
+  gtk_box_pack_start_defaults(GTK_BOX(vbox), d.time_check);
+  gtk_signal_connect(GTK_OBJECT(d.time_check), "toggled",
+                     GTK_SIGNAL_FUNC(file_preferences_changed), dialog);
+
+  label = gtk_label_new("Updating");
+  gnome_property_box_append_page(GNOME_PROPERTY_BOX(dialog), vbox, label);
+
+  gtk_signal_connect(GTK_OBJECT(dialog), "apply",
+                     GTK_SIGNAL_FUNC(file_preferences_apply), &d);
+
+  gtk_widget_show_all(dialog);
+  gnome_dialog_run_and_close(GNOME_DIALOG(dialog));
+
+  gtk_object_unref(GTK_OBJECT(d.zsub_check));
+  gtk_object_unref(GTK_OBJECT(d.time_check));
+  gtk_object_unref(GTK_OBJECT(d.time_interval));
+}
+
+/**
+ * file_preferences_changed - file_preferences callback for when a
+ * widget state changes
+ */
+void file_preferences_changed(GtkWidget *widget, gpointer data)
+{
+  gnome_property_box_changed(GNOME_PROPERTY_BOX(data));
+}
+
+/**
+ * file_preferences_apply - file_preferences dialog callback
+ */
+void file_preferences_apply(GnomePropertyBox *propbox,
+                            gint page, gpointer data)
+{
+  int ttime;
+  char *text, *text2;
+  struct file_preferences_data *d = data;
+  prefs.do_zsubs =
+    gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(d->zsub_check));
+  prefs.do_time =
+    gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(d->time_check));
+  text = gtk_entry_get_text(GTK_ENTRY(d->time_interval));
+  ttime = strtol(text, &text2, 10);
+  if (*text2 == '\0' && ttime > 0)
+    prefs.time_interval = ttime;
+  update_setup();
+  write_settings();
 }
 
 /**
@@ -329,3 +437,41 @@ void help_about(gpointer data)
   gtk_widget_show(widget);
   gnome_dialog_run_and_close(GNOME_DIALOG(widget));
 }
+
+void read_settings(void)
+{
+  prefs.do_zsubs = gnome_config_get_int("/" MYNAME "/settings/do_zsubs=1");
+  prefs.do_time = gnome_config_get_int("/" MYNAME "/settings/do_time=1");
+  prefs.time_interval =
+    gnome_config_get_int("/" MYNAME "/settings/time_interval=60");
+  update_setup();
+}
+
+void write_settings(void)
+{
+  gnome_config_set_int("/" MYNAME "/settings/do_zsubs", prefs.do_zsubs);
+  gnome_config_set_int("/" MYNAME "/settings/do_time", prefs.do_time);
+  gnome_config_set_int("/" MYNAME "/settings/time_interval",
+                       prefs.time_interval);
+  gnome_config_sync();
+}
+
+void update_setup(void)
+{
+  if (prefs.timer)
+  {
+    gtk_timeout_remove(prefs.timer);
+    prefs.timer = 0;
+  }
+  
+  if (prefs.do_time)
+    prefs.timer = gtk_timeout_add(prefs.time_interval * 1000,
+                                  update_timeout,
+                                  NULL);
+}
+
+gint update_timeout(gpointer data)
+{
+  znol_update();
+  return TRUE;
+}
